#!/usr/bin/python2.5 -u

import time
import rpyc

# Register an instance; all the methods of the instance are
# published as XML-RPC methods (in this case, just 'div').
import updb

class RpycUPdb_mixin(updb.UPdb_mixin):
    def exposed_debug_func(self, f, *o, **k):
        return self.debug_func(f, *o, **k)
    def exposed_undebug_func(self, f):
        return self.undebug_func(f)

import sys
class MyFuncs(RpycUPdb_mixin, rpyc.Service):
    def on_connect(self):
        # code that runs when a connection is created (to init the serivce, if needed)
        pass
 
    def on_disconnect(self):
        # code that runs when the connection has already closed (to finalize the service, if needed)
        pass

    def exposed_add(self, x, y):
        return x+y
    def exposed_pow(self, x, y):
	    return pow(x,y)
    def exposed_div(self, x, y):
        return x // y

    def exposed_slowprint(self, s):
        for c in s:
            sys.stdout.write(c)
            time.sleep(0.25)
        sys.stdout.write('\n')

    def exposed_longrun(self):
        while 1:
          time.sleep(0.5)


if __name__ == "__main__":
    from rpyc.utils.server import ThreadedServer
    t = ThreadedServer(MyFuncs, port = 18861)
    t.start()



import rpyc

s = rpyc.connect('localhost', 18861)
c = s.root

print c.pow(2,3)  # Returns 2**3 = 8
print c.add(2,3)  # Returns 5
print c.div(5,2)  # Returns 5//2 = 2

while True:
    c.slowprint('This is a test string')



#!/usr/bin/env python -u

import xmlrpclib
import time

s = xmlrpclib.ServerProxy('http://localhost:8000')
# Print list of available methods
print s.system.listMethods()

p = 0
while True:
    time.sleep(0.5)
    p = s.next_prime(p)
    print p



#!/usr/bin/python -u

# be compatible with Python 2.5
from __future__ import with_statement

import termsock, rpyc

def main(options):
    if options.funcname:
        c = rpyc.connect('localhost', 18861)
        s = c.root
        sock_name = s.debug_func(options.funcname, options.once, force=options.force)
    else:
        sock_name = options.sockname
    print sock_name

    dbg = termsock.TermSock(sock_name)
    dbg.mainloop()

if __name__ == '__main__':
    import optparse
    usage = "updb: connect to a UPdb debugger"
    parser = optparse.OptionParser(usage=usage)
    # TODO: consider a force option which removes any existing
    # unix socket
    parser.add_option("-s", "--sockname", dest="sockname", help="UNIX-domain socket path")
    parser.add_option("-a", "--funcname", dest="funcname", help="API function to debug")
    parser.add_option("-o", "--once", dest="once", action="store_false", help="Set breakpoint as once-only (default)", default=True)
    parser.add_option("-f", "--force", dest="force", action="store_true", help="Force (overwrite any previous socket)")
    options,args = parser.parse_args()
    if options.sockname and options.funcname:
        parser.error("Options --sockname and --funcname are mutually exlucsive")
    if not(options.sockname or options.funcname):
        parser.error("Must specify either --sockname or --funcname")

    main(options)



#!/usr/bin/python -u

# be compatible with Python 2.5
from __future__ import with_statement

import pdb, socket, sys, os, tempfile

class UPdb(pdb.Pdb):
    def __init__(self, sock_path, level=0, force=False):
        if force:
            try:
                os.remove(sock_path)
            except:
                pass
        self._sock_path = sock_path
        self._level = level
        self._sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        self._sock.bind(self._sock_path)
        self._sock.listen(1)
        self._conn = self._sock.accept()[0]
        self._handle = self._conn.makefile('rw')
        self._old_stdin = sys.stdin
        self._old_stdout = sys.stdout
        pdb.Pdb.__init__(self,
                         stdin=self._handle,
                         stdout=self._handle)
        sys.stdout = sys.stdin = self._handle

    def __enter__(self):
        target_frame = sys._getframe().f_back
        for count in range(self._level):
            try:
                target_frame = target_frame.f_back
            except AttributeError:
                break
        self.set_trace(target_frame)

    def __exit__(self, exc_type, exc_value, tb):
        sys.stdin = self._old_stdin
        sys.stdout = self._old_stdout

        self._conn.shutdown(socket.SHUT_RDWR)
        os.remove(self._sock_path)
        os.rmdir(os.path.dirname(self._sock_path))


class UPdb_mixin(object):
    """
    This is designed to be mixed-in to an RPC server
    object. It allows an RPC client to register a
    RPC method as being debuggable.

    On triggering, subsequent calls to that method
    will trigger a pdb breakpoint (currently with
    interaction over a unix socket)
    """
    # this is a map from function name to
    # (function, unix_socket_path) pairs.
    _updb_debug_func = {}

    def debug_func(self, f, once=True, force=True):
        """
        @param f: function name
        @param once: one-shot debug - undebug after first hit
        @param force: override any existing unix socket
        @return: path to debug socket
        """
        if f in self._updb_debug_func:
            return self._updb_debug_func[f][1]
        else:
            func = getattr(self, f)
            pdb_sock_path = "%s/pdb_sock"%tempfile.mkdtemp(prefix='updb_')
            self._updb_debug_func[f] = (func, pdb_sock_path)
            def _(*o, **k):
                if once:
                    self.undebug_func(f)
                with UPdb(pdb_sock_path, level=1, force=force):
                    return func(*o, **k)
            setattr(self, f, _)
            return pdb_sock_path

    def undebug_func(self, f):
        """
        don't debug this function anymore.
        """
        if f in self._updb_debug_func:
            setattr(self, f, self._updb_debug_func.pop(f)[0])
